;
;These routines were written to work on the pICDEM1 hardware.
; The frequency of the clock is 16Mhz and the hardware uses no
;handshake
;       TX -> RA3
;       RX -> RA2
;
;       Program:          SER54.ASM 
;       Revision Date:   
;                         12-12-95      Compatibility with MPASMWIN 1.30
;
;********************************************************************** 
;
        list p=16c54
;
clockrate equ .16000000         ;define clock rate here
baudrate equ    .9600           ;define baud rate here
;
fclk  equ     clockrate/4
;*************************************************************
;The value baudconst must be a 8 bit value only
baudconst       equ     ((fclk/baudrate)/3 - 2)
;*************************************************************
count   equ     0x10    ;used to count tx and rx data bits
txreg   equ     0x11    ;used as temp transmit register
rcreg   equ     0x11    ;received char is saved here
delay   equ     0x12    ;used to time the baud rate.
tempa   equ     0x13
hi      equ     0x10
lo      equ     0x11
gpram   equ     0x15
;
        include "p16c5x.inc"

#define         _ra1            PORTA,1

;
#define _tx     PORTA,3
#define _rx     PORTA,2
;
        org     0
start
        call    wait
        movlw   B'00001111'
        movwf   PORTA
        movlw   B'00000111'     ;RA3 = output, RA2 = input
        tris    PORTA
        clrf    PORTB          ;port b all outputs
        clrw
        tris    PORTB  
        movlw   0xa5            ;test of gpram check if... 
        xorwf   gpram,W         ;reset was POR or MCLR low
        btfsc   STATUS,Z              ;gpram = 0xa5?
        call    Mclr            ;yes then do Mclr routine
        movlw   0xa5            ;else load 0xar in gpram
        movwf   gpram
;
;
        movlw   0x2f
        movwf   tempa
        movlw   B'00111000'
        option
        clrf    TMR0
S1check
        movf    TMR0,W
        btfsc   STATUS,Z              ;if S1 pressed then skip
        goto    S1check
;
next
        incf    tempa, F
        btfsc   tempa,7
        goto    AllDone
        movf    tempa,W
        call    transmit
        call    receive
        subwf   tempa,W
        btfsc   STATUS,Z
        goto    fail
        goto    next
;
;
transmit
        movwf   txreg
        bcf     _tx             ;send start bit
        movlw   baudconst
        movwf   delay
        movlw   .9
        movwf   count
txbaudwait
        decfsz  delay, F
        goto    txbaudwait
        movlw   baudconst
        movwf   delay
        decfsz  count, F
        goto    SendNextBit
        movlw   .9
        movwf   count
        bsf     _tx             ;send stop bit
        return
SendNextBit
        rrf     txreg, F
        btfss   STATUS,C
        goto    Setlo
        bsf     _tx
        goto    txbaudwait
Setlo
        bcf     _tx
        goto    txbaudwait
;
;
receive
       btfsc    _rx
       goto     receive         ;wait for receive
rxbaudwait
        decfsz  delay, F
        goto    rxbaudwait
        movlw   baudconst
        movwf   delay
        decfsz  count, F
        goto    RecvNextBit
        movlw   .9
        movwf   count
        return
RecvNextBit
        bcf     STATUS,C
        btfsc   _rx
        bsf     STATUS,C
        rrf     rcreg, F
        goto    rxbaudwait
;
fail
        comf    PORTB, F
        call    halfsec
        goto    fail
halfsec
        clrf    hi
        clrf    lo
hsloop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        decfsz  lo, F
        goto    hsloop
        decfsz  hi, F
        goto    hsloop
        return
;
wait
        clrf    hi
        clrf    lo
dly
        decfsz  lo, F
        goto    dly
        decfsz  hi, F
        goto    dly
        return

;
AllDone
        movlw   0x55
        movwf   PORTB
        goto    $
;
Mclr
        movlw   0xaa
        movwf   PORTB
        clrf    gpram
        nop
S3check
        btfsc   _ra1
        goto    S3check
        movlw   0x55
        movwf   PORTB
        goto    $
;
;
        org     0x1ff
        goto    start
;
        end


       



